/* $Id: cmae_envelope.h,v 1.7 2006/08/09 16:11:36 chip Exp $ */

#ifndef _CLOUDMARK_AUTHORITY_ENGINE_ENVELOPE_H_
#define _CLOUDMARK_AUTHORITY_ENGINE_ENVELOPE_H_

#include "cmae_port.h"
#include "cmae_error.h"

#if __cplusplus
extern "C" {
#endif

/*
 * CMAE_Envelope - message delivery metadata associated with a message
 */

/*--- Object cookie ---*/
typedef void* CMAE_Envelope;

/* Create a new envelope object. You can specify NULL/0 for any items
 * that you do not wish to set. FromIP, if set, should be an IP address
 * in _network_ order. */
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeInit (CMAE_Envelope *EnvelopeOut,
                                        unsigned long FromIP,
                                        const char *FromHost,
                                        const char *HeloDomain,
                                        const char *MailFromAddress,
                                        const char **RcptTos,
                                        unsigned int NumRcptTos);

/*--- Free an envelope object */
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeDone    (CMAE_Envelope  Envelope);

/*--- Accessors. All strings point into the envelope object and do not need to be explicitly freed. ---*/

/* Note: this returns an IP address in _network_ order */
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeGetFromIP    (const CMAE_Envelope Envelope, unsigned long *FromIPOut);
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeGetFromHost  (const CMAE_Envelope Envelope, const char **FromHostOut);
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeGetHelo      (const CMAE_Envelope Envelope, const char **HeloDomainOut);
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeGetMailFrom  (const CMAE_Envelope Envelope, const char **EmailAddressOut);
CMAE_EXPORT CMAE_ERR CMAE_EnvelopeGetRcptTos   (const CMAE_Envelope Envelope, const char ***RcptTosOut, unsigned int *NumRcptTosOut);

#if __cplusplus
}
#endif

#endif /* _H_ */
