/* $Id: cmae_error.h,v 1.17 2006/09/14 17:42:37 bujak Exp $ */

#ifndef _CLOUDMARK_AUTHORITY_ENGINE_ERROR_H_
#define _CLOUDMARK_AUTHORITY_ENGINE_ERROR_H_

/*
 * CMAE error codes and related functionality.
 */

#include <limits.h>

#include "cmae_port.h"

#ifdef __cplusplus
extern "C" {
#endif

/*--- Error Codes ---*/
typedef enum {
    CMAE_ERR_NONE = 0,     /* no error */

    /* engine errors */

    CMAE_ERR_BADVERSION,  /* version number not supported by this engine */
    CMAE_ERR_BADARG,      /* argument passed to API function was missing or incorrect */
    CMAE_ERR_MALLOC,      /* out of memory */
    CMAE_ERR_CART_LOAD,   /* engine could not dynamically load the cartridge */
    CMAE_ERR_CART_INIT,   /* engine could not (re)initialize the cartridge */
    CMAE_ERR_INIT,        /* engine has already been initialized */
    CMAE_ERR_START,       /* engine has already been started */
    CMAE_ERR_NOINIT,      /* engine has not been initialized */
    CMAE_ERR_NOSTART,     /* engine has not been started */
    CMAE_ERR_CARTNOSUPP,  /* the associated cartridge does not support this operation */
    CMAE_ERR_SHUTDOWN,    /* the cartridge has been shutdown already */

    /* configuration errors */

    CMAE_ERR_CFG_WARNING,      /* configuration warning in user-specified param */
    CMAE_ERR_CFG_ERROR,        /* error in configuration */
    CMAE_ERR_CFG_KEY,          /* unknown or NULL key */
    CMAE_ERR_CFG_SYNTAX,       /* config parameter has syntax error in value */
    CMAE_ERR_CFG_VALUE,        /* config parameter has a bad value */
    CMAE_ERR_CFG_NULL,         /* config can't take a NULL value to 'unset' the value */
    CMAE_ERR_CFG_IMMUTABLE,    /* config parameter can't be changed (immutable) */
    CMAE_ERR_CFG_DEPRECATED,   /* config parameter is deprecated */

    /* client/server errors */

    CMAE_ERR_UNKNOWNHOST,  /* no such host [gethostbyname failure] */
    CMAE_ERR_PROTOCOL,     /* protocol error [unexpected data (not) received] */
    CMAE_ERR_MISCSOCKET,   /* miscellaneous socket error [catch-all] */
    CMAE_ERR_NOTCONN,      /* not connected [ENOTCONN] */
    CMAE_ERR_CONNREFUSED,  /* connection refused [ECONNREFUSED] */
    CMAE_ERR_NETUNREACH,   /* network is unreachable [ENETUNREACH] */
    CMAE_ERR_TIMEDOUT,     /* operation timed out [ETIMEDOUT, etc.] */
    CMAE_ERR_AGAIN,        /* temporary resource exhaustion [EAGAIN] */
    CMAE_ERR_CONNRESET,     /* connection reset by peer [ECONNRESET] */

    /* Generic error not covered by any of the above error values.
       This error value only exists for forwards compatibility for
       future releases that may introduce new functionality (and
       introduce new error values). */
    CMAE_ERR_OTHER,

    /* To prevent compilers from optimizing the size of CMAE_ERR to less than sizeof(int) */
    CMAE_ERR_UNUSED = INT_MAX
} CMAE_ERR;

/* CMAE_StrError() for all non-zero return values */
CMAE_EXPORT const char *CMAE_StrError(CMAE_ERR Error);


#if __cplusplus
}
#endif

#endif /* _H_ */
